%%    Favago Transport Programming Interface - Decimation
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%
%   Returns function handle to reduce the number of sites in the Hamiltonian via decimation.
%%
function ret = Decimation(Opt, varargin)

p = inputParser;
p.addParamValue('ForcedDecimation', []);
p.parse(varargin{:});
        
ForcedDecimation = p.Results.ForcedDecimation;

if ~isempty(ForcedDecimation)
    Opt.Decimation = ForcedDecimation;
end        
        
if Opt.Decimation == 3    
    ret = @decimation_3;
elseif  Opt.Decimation == 1 || Opt.Decimation == 2
    ret = @decimation_1_and_2;
else
    warning('MATLAB:Wrong_Decimation_Parameter', 'Wrong Decimation parameter given! Using default 3.')
    ret = @decimation_3;
end

%------------------------------------------------------------------
% Nested functions

%% decimation_3
%   Algorithm to reduce the number of sites in the Hamiltonian via decimation. The size of the decimated blocks is fixed. 
    function decimation_3(E, Way2Hamiltonian, NameOfH, NameOfKulso_szabfokok )
    
    	if Way2Hamiltonian.Read('HamiltoniansDecimated')
    		display('The Hamiltonian is already decimated',1);
    		return
        elseif ~Way2Hamiltonian.Read('HamiltoniansCreated')
        	display('The Hamiltonian needs to be created before the decimation',1);
        	return
       	end
        
        %blokk_size_max = 50;
        blokk_size_max = Opt.Decimation_block;
        
        Hamiltoni       = Way2Hamiltonian.Read( NameOfH );
        kulso_szabfokok = Way2Hamiltonian.Read( NameOfKulso_szabfokok );
        Way2Hamiltonian.Clear( NameOfKulso_szabfokok );
        
        setCoordinates( Way2Hamiltonian, kulso_szabfokok );
        
        megmaradt_szabfokok = szabfokok_decimalas_utan(kulso_szabfokok);
        Way2Hamiltonian.Write( NameOfKulso_szabfokok, megmaradt_szabfokok );
        
        Way2Hamiltonian.Clear( NameOfH );
        
        H_size          = size(Hamiltoni ,1);
        blokkok         = kulso_szabfokok_felosztasa();
        belso_szabfokok = find(blokkok{2} == 'b');
        
        whos_H          = whos('Hamiltoni');
        if whos_H.size(1)*whos_H.size(2)*16 <= whos_H.bytes && whos_H.sparse == 1
            Hamiltoni = full(Hamiltoni);
            display('A Hamltoni mar nem ritka');
        end 

        Idx            = length(belso_szabfokok);
        iidx           = 1;
        
        while iidx <= Idx
            idx = belso_szabfokok(iidx);
            decimation_of_block();
            
            whos_H = whos('Hamiltoni');
            if whos_H.size(1)*whos_H.size(2)*16 <= whos_H.bytes && whos_H.sparse == 1
                Hamiltoni = full(Hamiltoni);
                display('A Hamltoni mar nem ritka');
            end 
            
            iidx = iidx + 1;
        end
        
        
                
        Way2Hamiltonian.Write( NameOfH, Hamiltoni );
        Way2Hamiltonian.Write('HamiltoniansDecimated', 1)
        
        %------------------------------------------------------------------
        function decimation_of_block()
            
            oszlop  = [sum(blokkok{1}(1:idx-1))+1,sum(blokkok{1}(1:idx))];
            nevezo  = E*eye(blokkok{1}(idx)) - Hamiltoni(oszlop(1):oszlop(2),oszlop(1):oszlop(2));
            cond_number = rcond(nevezo); 
            tolerance   = 1e-4;
            
            %if abs(cond_number-1) > tolerance
            if cond_number < tolerance
                clear('nevezo'); 
                [oszlop, nevezo] = kondicios_szam_javito();
            end
            
            if cond_number < tolerance
                display(num2str(cond(nevezo)) )
            end
	
            [sor_idx,oszlop_idx] = find(Hamiltoni(oszlop(1):oszlop(2),:));
            oszlop_szamok        = oszlop_szamolo(oszlop_idx', oszlop, blokk_size_max);
            Kdx                  = size(oszlop_szamok,1);
            kdx                  = 1;
            while kdx <= Kdx
                oszlop_k  = oszlop_szamok(kdx,:); 
                jdx       = 1;
                H_darab21 = Hamiltoni(oszlop(1):oszlop(2),oszlop_k(1):oszlop_k(2));
                nevezo2   = nevezo\H_darab21;
                while jdx <= Kdx
                    oszlop_j  = oszlop_szamok(jdx,:);
                    H_darab11 = Hamiltoni(oszlop_j(1):oszlop_j(2),oszlop_k(1):oszlop_k(2));
                    H_darab12 = Hamiltoni(oszlop_j(1):oszlop_j(2),oszlop(1):oszlop(2));
                    H_darab11 = H_darab11 + H_darab12*nevezo2;
                    
                    Hamiltoni(oszlop_j(1):oszlop_j(2),oszlop_k(1):oszlop_k(2)) = H_darab11;
                    jdx       = jdx + 1;
                end
                kdx = kdx + 1;
            end 
        
            if issparse(Hamiltoni)
                [sor_idx,oszlop_idx,elemek] = find(Hamiltoni); 
                Hamiltoni           = [];
                indexek             = logical(((oszlop(1)<=sor_idx) & (sor_idx <=oszlop(2))) | ((oszlop(1)<=oszlop_idx) & (oszlop_idx <=oszlop(2))));
                sor_idx(indexek)    = [];
                oszlop_idx(indexek) = [];
                elemek(indexek)     = [];
                indexek             = logical(sor_idx>oszlop(2));
                sor_idx(indexek)    = sor_idx(indexek) - blokkok{1}(idx);
                indexek                  = logical(oszlop_idx>oszlop(2));
                oszlop_idx(indexek) = oszlop_idx(indexek) - blokkok{1}(idx);
                clear indexek;
                blokkok{1}(idx) = [];
                blokkok{2}(idx) = [];
                Hamiltoni       = sparse(sor_idx,oszlop_idx,elemek,sum(blokkok{1}),sum(blokkok{1}));
                clear sor_idx oszlop_idx elemek;
            else   
                Hamiltoni(oszlop(1):oszlop(2),:) = [];
                Hamiltoni(:,oszlop(1):oszlop(2)) = [];
                blokkok{1}(idx) = [];
                blokkok{2}(idx) = [];
                
            end                
                belso_szabfokok = belso_szabfokok -1;
                
                
            %------------------------------------------------------------------               
            % kiszamolja hogy a kidecimalando szabfokokat hogy kell beoszlopozni
            function oszlop_szamok = oszlop_szamolo(oszlop_idx, oszlop_decimalo, blokk_size_max)
    
            blokk_size_max = 150*blokk_size_max;
            ldx = 1;
            Ldx = length(oszlop_idx);

            if oszlop_idx(1) >= oszlop_decimalo(1)
                while oszlop_idx(ldx) <= oszlop_decimalo(2)
                    ldx = ldx + 1;
                end
                oszlop_szamok = [oszlop_idx(ldx),oszlop_idx(ldx)];    
            else
                oszlop_szamok = [oszlop_idx(1),oszlop_idx(1)];
            end

            ldx = ldx + 1;

            while (ldx <= Ldx) && (oszlop_decimalo(1) > oszlop_idx(ldx))
                if (oszlop_idx(ldx)-oszlop_szamok(end,1) > blokk_size_max)
                    oszlop_szamok(end,2) = oszlop_idx(ldx-1);
                    oszlop_szamok        = [oszlop_szamok;[oszlop_idx(ldx),oszlop_idx(ldx)]];       
                end    
                ldx = ldx + 1;
            end

            oszlop_szamok(end,2) = oszlop_idx(ldx-1);

            while (ldx < Ldx) && (oszlop_decimalo(2) > oszlop_idx(ldx))
                ldx = ldx + 1;
            end

            oszlop_szamok        = [oszlop_szamok;[oszlop_idx(ldx),oszlop_idx(ldx)]];

            while ldx <= Ldx
                if (oszlop_idx(ldx)-oszlop_szamok(end,1) > blokk_size_max)
                    oszlop_szamok(end,2) = oszlop_idx(ldx-1);
                    oszlop_szamok        = [oszlop_szamok;[oszlop_idx(ldx),oszlop_idx(ldx)]];     
                end
                ldx = ldx + 1;
            end

            oszlop_szamok(end,2) = oszlop_idx(ldx-1);
            %oszlop_szamok = [1,oszlop_decimalo(1)-1;oszlop_decimalo(2)+1,H_size];

            end
            %------------------------------------------------------------------
        
        
            %------------------------------------------------------------------
            function [oszlop,nevezo] = kondicios_szam_javito()
        
            %display(['condition number = ',num2str(cond_number)]); 
            blokkok{1} = [blokkok{1}(1:idx);blokkok{1}(idx:end)];
            blokkok{2} = [blokkok{2}(1:idx);blokkok{2}(idx:end)];
            egyik_fele = blokkok{2}(idx);
            blokkok{1}(idx+1) = 0;
            
            %while abs(cond_number-1) >= tolerance && egyik_fele >= 2
            while cond_number <= tolerance && egyik_fele >= 2
                egyik_fele        = round(blokkok{1}(idx)*2.99/4);
                masik_fele        = blokkok{1}(idx) - egyik_fele;
                blokkok{1}(idx)   = egyik_fele;
                blokkok{1}(idx+1) = masik_fele + blokkok{1}(idx+1);
                oszlop            = [sum(blokkok{1}(1:idx-1))+1,sum(blokkok{1}(1:idx))];
                nevezo            = E*eye(blokkok{1}(idx)) - Hamiltoni(oszlop(1):oszlop(2),oszlop(1):oszlop(2));
                cond_number       = rcond(nevezo);
            end
            %display(num2str(blokkok{1}(idx)))
            %display(['atjavitott condition number = ',num2str(cond_number),' valamint kidecimalando szabfokok = ',num2str(blokkok{1}(idx))]);
            belso_szabfokok = [belso_szabfokok(1:iidx);belso_szabfokok(iidx:end)];
            belso_szabfokok(iidx+1:end) = belso_szabfokok(iidx+1:end) + 1;
            Idx = length(belso_szabfokok);
        
        
        
            end
            %------------------------------------------------------------------
                
                
        end
        %------------------------------------------------------------------
        
        
        %------------------------------------------------------------------
        % Kiszamolja a belso es kulso szabadsagi fokok osszefuggo blokkjait
        function blokkok = kulso_szabfokok_felosztasa()
        
        
            kulso_szabfokok = sort(kulso_szabfokok);
            iii_max         = length(kulso_szabfokok);
            iiidx           = 1;
            blokkok         = cell(2,1);
        
            while iiidx <= iii_max
            
                szabfok_tipus = [];
                blokk_size    = 1;
                if (sum(blokkok{1}) + blokk_size) == kulso_szabfokok(iiidx)
                    while (iiidx == iii_max || kulso_szabfokok(iiidx+1) == kulso_szabfokok(iiidx)+1) && blokk_size <= blokk_size_max
                        blokk_size = blokk_size + 1;
                        iiidx      = iiidx + 1;
                        if iiidx >= iii_max; blokk_size = blokk_size-1; break; end
                    end
                    if blokk_size == 1 && iiidx < iii_max ; iiidx = iiidx + 1; end
                    szabfok_tipus = 'k';  % kulso szabfok 
                elseif (sum(blokkok{1}) + blokk_size) < kulso_szabfokok(iiidx)
                    while (sum(blokkok{1}) + blokk_size + 1 < kulso_szabfokok(iiidx)) && blokk_size <= blokk_size_max
                        blokk_size = blokk_size + 1;
                    end                
                    szabfok_tipus = 'b'; % kidecimalando belso szabfokok
                elseif isempty(szabfok_tipus) && (sum(blokkok{1}) + blokk_size) > kulso_szabfokok(iiidx)
                    iiidx = iiidx + 1;
                    szabfok_tipus = [];
                    blokk_size    = [];
                end
            
                blokkok{1} = [blokkok{1};blokk_size];
                blokkok{2} = [blokkok{2};szabfok_tipus];
     
            
            end
        
            szabfokok = H_size-sum(blokkok{1});
        
            if szabfokok > 0
                blokk_size_max     = blokk_size_max + 1;
                blokkok_kieg       = ones((szabfokok-mod(szabfokok,blokk_size_max))/blokk_size_max,1)*blokk_size_max;
                szabfok_tipus_kieg = char(ones((szabfokok-mod(szabfokok,blokk_size_max))/blokk_size_max,1)*'b');
                blokkok{1}         = [blokkok{1};blokkok_kieg;mod(szabfokok,blokk_size_max)];
                blokkok{2}         = [blokkok{2};szabfok_tipus_kieg;'b'];
            end
        
        end
        %------------------------------------------------------------------
        
        
        %------------------------------------------------------------------
        % Meghatarozza a megmaradt szabadsagi fokokat
        function megmaradt_szabfokok = szabfokok_decimalas_utan(kulso_szabfokok)
        
            L          = length(kulso_szabfokok);
            idx        = 1;
            ertek_temp = [];
            Maximum    = max(kulso_szabfokok);
            megmaradt_szabfokok = zeros(L,1);
            while idx <= L
                szabfok_max   = max(megmaradt_szabfokok);
                [ertek,index] = min(kulso_szabfokok);
                if ertek_temp == ertek
                    megmaradt_szabfokok(index) = szabfok_max;
                else
                    ertek_temp    = ertek;
                    megmaradt_szabfokok(index) = szabfok_max+1;
                end
                kulso_szabfokok(index) = Maximum+1;
                idx = idx + 1;
            end
    
    
   
        end
        %------------------------------------------------------------------       
    
        
        
    end

% setting coordinates
    function setCoordinates( Way2Hamiltonian, kulso_szabfokok )
        
        coordinates     = Way2Hamiltonian.Read( 'coordinates' );
        
        if isempty(coordinates)
            display('structure coordinates is empty!')
            return
        end
        
        indexes = logical( kulso_szabfokok <= length(coordinates.x) );
        
        coordinates.x     = coordinates.x(kulso_szabfokok(indexes));
        coordinates.y     = coordinates.y(kulso_szabfokok(indexes));   
        
        if isfield( coordinates, 'spinup' ) && ~isempty( coordinates.spinup )
            coordinates.spinup = coordinates.spinup(kulso_szabfokok(indexes));
        end
        
        if isfield( coordinates, 'BdG_u' ) && ~isempty( coordinates.BdG_u )
            coordinates.BdG_u = coordinates.BdG_u(kulso_szabfokok(indexes));
        end
        
        Way2Hamiltonian.Write('coordinates', coordinates );        
        
    end
        

%% decimation_1_and_2
%   Algorithm to reduce the number of sites in the Hamiltonian via decimation. The size of the decimated blocks is dynamically varied to avoid badly conditioned matrices. 
function decimation_1_and_2(E, Way2Hamiltonian, NameOfH, NameOfKulso_szabfokok )

    if Way2Hamiltonian.Read('HamiltoniansDecimated')
    	display('The Hamiltonian is already decimated',1);
    	return
    elseif ~Way2Hamiltonian.Read('HamiltoniansCreated')
       	display('The Hamiltonian needs to be created before the decimation',1);
       	return
    end
    
    Hamiltoni       = Way2Hamiltonian.Read( NameOfH );
    kulso_szabfokok = Way2Hamiltonian.Read( NameOfKulso_szabfokok );
    Way2Hamiltonian.Clear( NameOfH );
    Way2Hamiltonian.Clear( NameOfKulso_szabfokok );
    
    setCoordinates( Way2Hamiltonian, kulso_szabfokok );
    
    Idx = size(Hamiltoni, 1);
    idx = 1;
    [minidx,minkoord]   = min(kulso_szabfokok);
    minidxtemp          = 0;
    szabfokok_szama     = length(kulso_szabfokok); 
    megmaradt_szabfokok = zeros(szabfokok_szama,1);



    while idx <= Idx
    
        if idx < minidx
            if Opt.Decimation == 1
                decimalas_pontonkent;
                kulso_szabfokok   = kulso_szabfokok - 1;
                minidx            = minidx - 1;
                Idx               = size(Hamiltoni, 1);
            elseif Opt.Decimation == 2
                blokk_size_max    = Opt.Decimation_block;
                blokk_size        = minidx - idx;
                if blokk_size > blokk_size_max
                    blokk_size = blokk_size_max;
                end
                decimalas_blokkonkent(blokk_size);
                s = whos('Hamiltoni');
                if s.size(1)*s.size(2)*16 <= s.bytes && s.sparse == 1
                    Hamiltoni = full(Hamiltoni);
                else
                    Hamiltoni = sparse(Hamiltoni);
                end
                kulso_szabfokok   = kulso_szabfokok - blokk_size;
                minidx            = minidx - blokk_size;
                Idx               = size(Hamiltoni, 1);
            else
                error('Hibas decimalasi parameter: Opt_Decimation');
            end       
        
        else
            if minidx ~= minidxtemp
                idx = idx + 1;
            end   
  
            if minidx <= Idx
                megmaradt_szabfokok(minkoord) = idx-1;
                kulso_szabfokok(minkoord)     = Idx+1;
            end
                minidxtemp        = minidx;
                [minidx,minkoord] = min(kulso_szabfokok);
        end    

      
    end

    if megmaradt_szabfokok(minkoord) == 0
        megmaradt_szabfokok(minkoord) = minidx;
    end
    
    Way2Hamiltonian.Write( NameOfH, Hamiltoni );
    Way2Hamiltonian.Write( NameOfKulso_szabfokok, megmaradt_szabfokok );
    Way2Hamiltonian.Write( 'HamiltoniansDecimated', 1 );
    
    %------------------------------------------------------------------
    function decimalas_blokkonkent(blokk_size)


        Hsize = Idx;

        E_matrix = eye(blokk_size)*E;

        if idx == 1
            Hamiltoni = Hamiltoni(blokk_size+1:Hsize,blokk_size+1:Hsize) + ...
                Hamiltoni(blokk_size+1:Hsize,1:blokk_size) /(E_matrix-Hamiltoni(1:blokk_size,1:blokk_size) * Hamiltoni(1:blokk_size,blokk_size+1:Hsize));
    
        elseif idx+blokk_size-1 == Hsize
    
            Hamiltoni = Hamiltoni(1:Hsize-blokk_size,1:Hsize-blokk_size) + ...
                Hamiltoni(1:Hsize-blokk_size,Hsize-blokk_size+1:Hsize) /(E_matrix-Hamiltoni(Hsize-blokk_size+1:Hsize,Hsize-blokk_size+1:Hsize)) * Hamiltoni(Hsize-blokk_size+1:Hsize,1:Hsize-blokk_size);
       
        else
            blokk_size = blokk_size-1;
            nevezo  = (E_matrix-Hamiltoni(idx:idx+blokk_size,idx:idx+blokk_size))\eye(size(E_matrix));
            Hdeci11 = Hamiltoni(1:idx-1,1:idx-1) + Hamiltoni(1:idx-1,idx:idx+blokk_size)*nevezo*Hamiltoni(idx:idx+blokk_size,1:idx-1);
            Hdeci12 = Hamiltoni(1:idx-1,idx+blokk_size+1:Hsize) + Hamiltoni(1:idx-1,idx:idx+blokk_size)*nevezo*Hamiltoni(idx:idx+blokk_size,idx+blokk_size+1:Hsize);
            Hdeci21 = Hamiltoni(idx+blokk_size+1:Hsize,1:idx-1) + Hamiltoni(idx+blokk_size+1:Hsize,idx:idx+blokk_size)*nevezo*Hamiltoni(idx:idx+blokk_size,1:idx-1);
            Hdeci22 = Hamiltoni(idx+blokk_size+1:Hsize,idx+blokk_size+1:Hsize) + Hamiltoni(idx+blokk_size+1:Hsize,idx:idx+blokk_size)*nevezo*Hamiltoni(idx:idx+blokk_size,idx+blokk_size+1:Hsize);
            Hamiltoni = [];
            Hamiltoni = [Hdeci11,Hdeci12;Hdeci21,Hdeci22];
        end
        
    end
    %------------------------------------------------------------------
    
    
    %------------------------------------------------------------------
    % decimalas pontonkent
    function decimalas_pontonkent()%(varname,E,idx)
        
        
        
        
        nem_nulla_idx = find(Hamiltoni(:,idx));
        rekurzioszam  = max(size(nem_nulla_idx));

        sor    = 1;
        while sor <= rekurzioszam
       
            if ~(nem_nulla_idx(sor) == idx)
                oszlop = 1;
                while oszlop <= rekurzioszam
                    if nem_nulla_idx(oszlop) == idx
                        oszlop = oszlop + 1;
                    else
                        Hamiltoni(nem_nulla_idx(sor),nem_nulla_idx(oszlop)) = Hamiltoni(nem_nulla_idx(sor),nem_nulla_idx(oszlop)) + ...
                            Hamiltoni(nem_nulla_idx(sor),idx)*Hamiltoni(idx,nem_nulla_idx(oszlop)) / (E-Hamiltoni(idx,idx));
                        oszlop = oszlop + 1;
                    end
                end
            end
            sor = sor + 1; 
        end
        Hamiltoni(idx,:) = [];
        Hamiltoni(:,idx) = [];

    end
    %------------------------------------------------------------------        
            
    end 
    
    

        
% displays output messages on the screen
    function display( message, nosilent )
        if ~Opt.Silent | exist('nosilent', 'var')
            disp( message )
        end
       
        if Opt.debug
            fid = fopen('debug.txt', 'a');
            fprintf(fid, [message,'\n']);
            fclose(fid);
        end
        
    end



%end of Nested functions
%--------------------------------------------------------------------------
end % Global end
    
    
    
   
    
  
