<?php $object_name = CombineRibbons; ?>
<h1 class="methods" class="methods"><?php echo $object_name; ?></h1>

<?php include ('licence.html'); ?>
  <p class="caption">Description</p>
  <p class="syntax">Returns an interface to combine multiple ribbons of equal width from the same material.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface = CombineRibbons(varargin)</p>
        
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">filenameIn</p>Input filename for the xml input structure.</li>
      <li><p class="strucfield">filenameOut</p>Output filename for the xml input structure.</li>
      <li><p class="strucfield">E</p>The energy used in the calculations.</li>
      <li><p class="strucfield">EF</p>The Fermi energy in eV.</li>
      <li><p class="strucfield">Ribbons</p>The list of interface <a href="index.php?location=Ribbon&menu=interfaces">Ribbon</a> instances to be combined.</li>
      <li><p class="strucfield">silent</p>Set true for suppress the displaying of the output messages.</li>
  </ul>
</td>
</tr>
</table>


<p class="caption">Attributes</p>
<ul>
	<li><?php $method_name = "WS" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> An instance of the class handle <a href="index.php?menu=structures&location=ws_CombineRibbons" >ws_CombineRibbons</a>.</li>
	<li><?php $method_name = "ribbon" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the interface <a href="index.php?location=Ribbon&menu=interfaces">Ribbon</a> to be used in transport calculations through the combined ribbon structure.</li>
	<li><?php $method_name = "WS_ribbon" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the class handle <a href="index.php?menu=structures&location=ws_Ribbon" >ws_Ribbon</a> related to the attribute ribbon.</li>
	<li><?php $method_name = "gfin" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The surface Greens function of the combined ribbons.</li>
	<li><?php $method_name = "gfininv" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The inverse of the surface Greens function of the combined ribbons.</li>
	<li><?php $method_name = "coordinates" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the structure <a href="index.php?menu=structures&location=Structures#coordinates" >coordinates</a> containing the coordinates of the surface sites of the combined ribbon structure.</li>
	<li><?php $method_name = "CommonFunctionsHandle" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the interface <a href="index.php?location=CommonFunctions&menu=interfaces" >CommonFunctions</a>.
</ul>


<p class="caption">Methods</p>
<ul>
	<li><?php $method_name = "Transport" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "create_combined_GreensFunction" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "addRibbon" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "getGfininv" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "getTotalHeight" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "getHeights" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "setHandlesForMagneticField" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "GetWorkspace" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
</ul>

<?php $method_name = "Transport" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the conductance through the combined ribbon strucutre. Use for development purpose only.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> [Conductance,ny,DeltaC] = interface.<?php echo $method_name; ?>(Energy) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to be used in the calculations.
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
Conductance
</td>
<td class="parameter_values">
The calculated conductance in the units of $$2h/e$$.
</td>
</tr>
<tr>
<td class="parameter_names">
ny
</td>
<td class="parameter_values">
The number of the open channels in the leads.
</td>
</tr>
<tr>
<td class="parameter_names">
DeltaC
</td>
<td class="parameter_values">
The standard deviation of the calculated transmission probabilities.
</td>
</tr>
</table>
  <br></br><br></br><br></br>



<?php $method_name = "create_combined_GreensFunction" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the surface Greens function of the combined ribbon structure. The calculated surface Greens function or the inverse of the Greens function are stored in the attributes <a href="#gfin">gfin</a> and <a href="#gfininv">gfininv</a>, respectively.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( varargin ) </p>
  
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">E</p>The energy to be used in the calculations. If not given, then the previously set energy is used in the calculations.</li>
      <li><p class="strucfield">onlygfininverz</p>Logical value. Set true to calculate only the inverse of the surface Greens function <a href="#gfininv">gfininv</a>, or false (default) to calculate <a href="#gfin">gfin</a> as well. In the latter case the attribute <a href="#gfininv">gfininv</a> is set to empty at the end.</li>
  </ul>
</td>
</tr>
</table>  

  <br></br><br></br><br></br>





<?php $method_name = "addRibbon" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Use this function to add another ribbon to the combined ribbon structure.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( ribbon2add, varargin  ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
ribbon2add
</td>
<td class="parameter_values">
An instance of the interface <a href="index.php?location=Ribbon&menu=interfaces">Ribbon</a> to be added to the ribbon structure.
</td>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">shiftcoordinates</p>Logical value. Set true (default) to automatically shift the coordinates of the interface <a href="index.php?location=Ribbon&menu=interfaces">ribbon2add</a> to match the corrdinates of the <a href="index.php?location=Ribbon&menu=interfaces">ribbon</a> interfaces listed in the attribute <a href="index.php?menu=structures&location=ws_CombineRibbons#Ribbons">Ribbons</a>.</li>
  </ul>
</td>
</tr>
</table>
  <br></br><br></br><br></br>





<?php $method_name = "getGfininv" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Reads out the attribute <a href="gfininv">gfininv</a>.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> ret = interface.<?php echo $method_name; ?>( ) </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
The inverse Greens function <a href="gfininv">gfininv</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "getTotalHeight" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the total height of the combined ribbon structure, including the interface regions between the ribbons.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> ret = interface.<?php echo $method_name; ?>() </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
The total height (in units of the 1D lattice vector) of the combined ribbon structure.
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<?php $method_name = "getHeights" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Returns a vector containing the individual heights of the ribbons in the structure (without the interface regions between them).</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> ret = interface.<?php echo $method_name; ?>( ) </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
The vector containing the individual heights of the ribbons in the structure.
</td>
</tr>
</table>
  <br></br><br></br><br></br>





<?php $method_name = "setHandlesForMagneticField" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Sets the function handles of the vector potentials and gauge fields for applying the magnetic field in the <a href="index.php?location=Ribbon&menu=interfaces">ribbon</a> interfaces listed in the attribute <a href="index.php?menu=structures&location=ws_CombineRibbons#Ribbons">Ribbons</a>.</p>
<p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>( vavargin ) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">scatter</p>Function handle of the vector potential to be used in the scattering region.</li>
      <li><p class="strucfield">lead</p>Function handle of the vector potential to be used in the leads.</li>
      <li><p class="strucfield">gauge_field</p>Function handle of the gauge transformation.</li>
  </ul>
</td>
</tr>
</table>
  <br></br><br></br><br></br>




<?php $method_name = "GetWorkspace" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Return the handle class <a href="index.php?location=ws_Ribbon&menu=structures">ws_ribbon</a> of the workspace of the interface <a href="index.php?location=Ribbon&menu=interfaces">Ribbon</a>. Changes in fields of the workspace are not always safe, since the reinitialization of the interface is not performed.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>(  ) </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
An instance of handle class <a href="index.php?location=ws_Ribbon&menu=structures">ws_ribbon</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>


