<?php $object_name = Peierls; ?>
<h1 class="methods" class="methods"><?php echo $object_name; ?></h1>

<?php include ('licence.html'); ?>

  <p class="caption">Description</p>
  <p class="syntax">Returns an interface to perform the Peierls and the gauge transformations on the Greens functions and on the Hamiltonians.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">object = Peierls(Opt, varargin)</p>
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Opt
</td>
<td class="parameter_values">
Structure containing the computational parameters. The required fields of the structure are:
  <ul>
      <li><p class="strucfield">BdG</p> Set true for Bogoliubov de Gennes Hamiltonian.></li>
      <li><p class="strucfield">my_own_potential</p> Set 0 to load the magnetic field from B_input.mat, or 1 to load the vector potential from Vector_Potetntial.mat (Obsolete).</li>
      <li><p class="strucfield">Linear_Regression_in_B</p>Set true for homogeneous magnetic field, false otherwise.</li>
      <li><p class="strucfield">Lattice_Type</p> set 'H' for hexagonal lattice, 'S' for square lattice, 'Silicene' for silicene lattice.</li>
      <li><p class="strucfield">type_of_scanning</p> if set to 1, the Peierls phases are loaded from the interface <a href="index.php?menu=interfaces&location=Surface_Green_function">Surface_Green_function</a> or <a href="index.php?menu=interfaces&location=CreateHamiltoninans">CreateHamiltoninans</a>.</li>
  </ul>
See more details about the data structure Opt <a href="index.php?menu=structures&location=Structures#Opt">here</a>.
</td>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameters are the following:
  <ul>
      <li><p class="strucfield">parameter</p>      scalar parameter to scale the strength of the magnetic field if Scanning.type_of_scanning=1</li>
      <li><p class="strucfield">Vectorpotential</p> function handle of the vector potential $$ [Ax,Ay] = A(x,y) $$</li>
      <li><p class="strucfield">gauge</p>          set 'Landaux' or 'Landauy' to use the in-built vector potentials.</li>
      <li><p class="strucfield">eta</p>            scalar value encoding the strength of the magnetic field if parameter 'gauge' is set (and 'Vectorpotential' is not set).</li>
  </ul>
</td>
</tr>
</table>



<p class="caption">Attributes</p>
<ul>
	<li><?php $method_name = "parameter" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The Peierls phases in the Hamiltonian can be multiplied by this parameter.</li>
	<li><?php $method_name = "gauge" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The name of the built-in gauge ('LandauX', 'LandauY').</li>
	<li><?php $method_name = "eta" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The strength of the magnetic field for the built-in vector potentials.</li>
	<li><?php $method_name = "Vectorpotential" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The function handle of the vector potential.</li>
	<li><?php $method_name = "Ax" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>Obsolete.</li>
	<li><?php $method_name = "Ay" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>Obsolete.</li>
	<li><?php $method_name = "B_input" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>Obsolete</li>
	<li><?php $method_name = "Opt" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the structure <a href="index.php?menu=structures&location=Structures#Opt" >Opt</a></li>
</ul>



<p class="caption">Methods</p>
<ul>
	<li><a href="#3">PeierlsTransformLeads</a></li>
	<li><a href="#4">PeierlsTransform</a></li>
	<li><a href="#5">gaugeTransformation</a></li>
	<li><a href="#6">gaugeTransformationOnLead</a></li>
	<li><a href="#7">setVectorPotential</a></li>
    <li><?php $method_name = "CreateClone" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
</ul>

<h2 class="methods"><?php echo $object_name; ?>::PeierlsTransformLeads<a name="3"></a></h2>
<p class="caption">Description</p>
  <p class="syntax">Algorithm to perform the Peierls transformation in the leads.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">object.PeierlsTransformLeads( Surface_tmp, varargin )</p>
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Surface_tmp
</td>
<td class="parameter_values">
An instance of a <a href="index.php?menu=interfaces&location=Surface_Green_function">Surface_Green_function</a> interface.
</td>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Currently no optional parameters are supported.
</td>
</tr>
</table>
  <br></br><br></br><br></br>





<h2 class="methods"><?php echo $object_name; ?>::PeierlsTransform<a name="4"></a></h2>
<p class="caption">Description</p>
<p class="syntax">Algorithm to perform Peierls transformation in Hamiltonian stored in the Way2Hamiltonian object.</p>
  <p class="caption">Syntax</p>
<p class="syntax">object.PeierlsTransform( Way2Hamiltonian )</p>
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Way2Hamiltonian
</td>
<td class="parameter_values">
An instance of the interface <a href="index.php?menu=interfaces&location=CreateHamiltoninans">CreateHamiltoninans</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>




<h2 class="methods"><?php echo $object_name; ?>::gaugeTransformation<a name="5"></a></h2>
<p class="caption">Description</p>
<p class="syntax">Algorithm to perform the gauge transformation on a Greens function and/or on a Hamiltonian.</p>
  <p class="caption">Syntax</p>
<p class="syntax">object.gaugeTransformation( mtx, coordinates, gauge_field ) </p>
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
mtx
</td>
<td class="parameter_values">
The matrix of the Greens function or Hamiltonian to be gauge-transformed.
</td>
</tr>
<td class="parameter_names">
coordinates
</td>
<td class="parameter_values">
A structure <a href="index.php?menu=structures&location=Structures#coordinates">coordinates</a>containing the coordinates of the sites.
</td>
</tr>
<td class="parameter_names">
gauge_field
</td>
<td class="parameter_values">
A function handle of the scalar gauge field $$ e/hbar*A_new = e/hbar*A_old + grad( gauge_field ) $$.
</td>
</tr>
</table>
  <br></br><br></br><br></br>




<h2 class="methods"><?php echo $object_name; ?>::gaugeTransformationOnLead<a name="6"></a></h2>
<p class="caption">Description</p>
<p class="syntax">Algorithm to perform the gauge transformation on the Hamiltonians of a lead.</p>
  <p class="caption">Syntax</p>
<p class="syntax">object.gaugeTransformationOnLead( Surface_tmp, gauge_field ) </p>
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Surface_tmp
</td>
<td class="parameter_values">
An instance of the interface <a href="index.php?menu=interfaces&location=Surface_Green_function">Surface_Green_function</a>.
</td>
</tr>
<td class="parameter_names">
gauge_field
</td>
<td class="parameter_values">
For details see <a href="#3">gaugeTransformation</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>






<h2 class="methods"><?php echo $object_name; ?>::setVectorPotential<a name="7"></a></h2>
<p class="caption">Description</p>
<p class="syntax">Sets the function handle of the vector potential.</p>
  <p class="caption">Syntax</p>
<p class="syntax">object.setVectorPotential( vectorpot )</p>
<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
vectorpot
</td>
<td class="parameter_values">
The function handle of the vector potential $$ [Ax,Ay] = A(x,y) $$.
</td>
</tr>
</table>
  <br></br><br></br><br></br>


<?php $method_name = "CreateClone" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates a clone of the current interface.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface = interface.<?php echo $method_name; ?>() </p>


</br>
<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
interface
</td>
<td class="parameter_values">
An instance of interface <a href="index.php?location=Peierls&menu=interfaces">Peierls</a>.
</td>
</tr>
</table>
<br></br><br></br><br></br>
