<?php $object_name = antidot; ?>
<h1 class="methods" class="methods"><?php echo $object_name; ?></h1>

<?php include ('licence.html'); ?>
  <p class="caption">Description</p>
  <p class="syntax">Returns an interface to perform transport calculations on an antidot (i.e., a hollow in a ribbon).</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface = antidot(varargin)</p>

<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequence ...'name', value,... . Possible parameter names are the following:
  <ul>
      <li><p class="strucfield">width</p>Integer. Gives the number of the atomic sites in the cross section of the ribbon.</li>
      <li><p class="strucfield">height</p>Integer. Gives the height of the ribbon in units of the lattice vector.</li>
      <li><p class="strucfield">B</p>The strength of the magnetic field in the units of Tesla.</li>
      <li><p class="strucfield">hole</p>An instance of structure <a href="index.php?menu=structures&location=Structures#hole">hole</a>.</li>
      <li><p class="strucfield">scatterers</p>An instance of structure <a href="index.php?menu=structures&location=Structures#scatterers">scatterers</a>.</li>
      <li><p class="strucfield">radius</p>The radius of the hole in units of lattice constant. (Obsolete)</li>
      <li><p class="strucfield">PNpotential</p>The height of the pn potential across the ribbon.</li>
      <li><p class="strucfield">PNeta</p>The width of the pn potential step in units of the total width of the ribbon. ($$0 less PNeta less 1$$)</li>
      <li><p class="strucfield">filenameIn</p>Input filename for the xml input structure.</li>
      <li><p class="strucfield">filenameOut</p>Output filename for the xml input structure.</li>
      <li><p class="strucfield">silent</p>Set true for suppress the displaying of the output messages.</li>
  </ul>
</td>
</tr>
</table>


<p class="caption">Attributes</p>
<ul>
	<li><?php $method_name = "WS" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> An instance of class handle <a href="index.php?location=ws_antidot&menu=structures" >ws_antidot</a>.</li>
	<li><?php $method_name = "a1" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>A lattice vector in the hexagonal lattice.</li>
	<li><?php $method_name = "a2" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>A lattice vector in the hexagonal lattice.</li>
	<li><?php $method_name = "ribbon" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p> An instance of the interface <a href="index.php?location=Ribbon&menu=interfaces" >Ribbon</a>.</li>
	<li><?php $method_name = "WS_ribbon" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of class handle <a href="index.php?location=ws_Ribbon&menu=interfaces" >ws_Ribbon</a>.</li>
	<li><?php $method_name = "gfin" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The Greens function of the scattering region.</li>
	<li><?php $method_name = "gfininv" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The inverse of the Greens function of the scattering region.</li>
	<li><?php $method_name = "waveFncDirnameFull" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>Directory name to export the plotted wave functions.</li>
	<li><?php $method_name = "waveFncSubDirname" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>The name of the subdirectory to export the plotted wave functions.</li>
	<li><?php $method_name = "CommonFunctionsHandle" ?><p class="strucfield"><a name="<?php echo $method_name; ?>"><?php echo $method_name; ?></a></p>An instance of the interface <a href="index.php?location=CommonFunctions&menu=interfaces" >CommonFunctions</a>.
</ul>

<p class="caption">Exported interfaces</p>
<ul>
	<li><a href="index.php?location=Ribbon&menu=interfaces" >ribbon</a></li>
</ul>

<p class="caption">Methods</p>
<ul>
	<li><?php $method_name = "Transport" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "create_scatter_GreensFunction" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CreateHandlesForMagneticField" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "getHolePoints" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "create_hole_Hamiltonian" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "CalcWavefnc" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
	<li><?php $method_name = "GetWorkspace" ?><a href="#<?php echo $method_name; ?>"><?php echo $method_name; ?></a></li>
</ul>

<?php $method_name = "Transport" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the conductance of an antidot connected to the leads in the "contact/scattering center/contact" arrangement.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> [Conductance,ny,DeltaC] = interface.<?php echo $method_name; ?>(Energy) </p>


<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to be used in the calculations.
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
Conductance
</td>
<td class="parameter_values">
The calculated conductance in the units of $$2h/e$$.
</td>
</tr>
<tr>
<td class="parameter_names">
ny
</td>
<td class="parameter_values">
The number of the open channels in the leads.
</td>
</tr>
<tr>
<td class="parameter_names">
DeltaC
</td>
<td class="parameter_values">
The standard deviation of the calculated transmission probabilities.
</td>
</tr>
</table>
  <br></br><br></br><br></br>




<?php $method_name = "CalcWavefnc" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the energies and wave functions of the bound states for antidot and dot systems.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> ret = interface.<?php echo $method_name; ?>(Energy, closefigure, varargin) </p>

<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
Energy
</td>
<td class="parameter_values">
The energy to be used in the calculations.
</td>
</tr>
<tr>
<td class="parameter_names">
closefigure
</td>
<td class="parameter_values">
If true, the created figure is closed at the end of the calculations.
</td>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequance ...'name', value,... . Possible parameters are the following:
  <ul class="style2list">
     	<li><p class="strucfield">toPlot</p>Set true (default) for plotting the wave function, or false otherwise.</li>
		<li><p class="strucfield">Einhomegac</p>Set true if the Energy is given in units of $$\hbar\omega_c$$, or false (default) otherwise.</li>
     	<li><p class="strucfield">filterHole</p>Set true (default) for separating antidot bound states from the edge states, or false otherwise.</li>
     	<li><p class="strucfield">db</p>The number of the calculated energy eigenvalues. Default is $$db=30$$.</li>
		<li><p class="strucfield">infinitemass</p>Set true for using the infinite mass boundary condition, or false (default) otherwise.</li>
     	<li><p class="strucfield">dotCalc</p>Set true for a dot, or false (default) otherwise.</li>
		<li><p class="strucfield">delta</p>Every $$db$$th point of the wave function becomes plotted. Default is $$db=4$$.</li>
     	<li><p class="strucfield">smoothedge</p>Set true for using a smooth edge in the calculations, or false (default) otherwise.</li>
  </ul>
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
A structure containing the calculated results. The fields of the structure are the following:
  <ul class="style2list">
     	<li><p class="strucfield">eigvals</p>The list of the calculated energy eigenvalues.</li>
		<li><p class="strucfield">eigvecs</p>The list of the calculated wave functions.</li>
     	<li><p class="strucfield">xcoords</p>The meshgrid of the $$x$$ coordinates.</li>
     	<li><p class="strucfield">ycoords</p>The meshgrid of the $$y$$ coordinates.</li>
  </ul>
</td>
</tr>
</table>

  <br></br><br></br><br></br>





<?php $method_name = "create_hole_Hamiltonian" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates the Hamiltonian for the antidot/dot.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> [Hscatter, wavefunction_indexes] = interface.<?php echo $method_name; ?>( varargin ) </p>

<table>
<tr>
<th colspan="2"> Parameters </th>
</tr>
<tr>
<td class="parameter_names">
varargin
</td>
<td class="parameter_values">
Optional parameters given by a sequance ...'name', value,... . Possible parameters are the following:
  <ul class="style2list">
		<li><p class="strucfield">infinitemass</p>Set true for using the infinite mass boundary condition, or false (default) otherwise.</li>
     	<li><p class="strucfield">dotCalc</p>Set true for a dot, or false (default) otherwise.</li>
     	<li><p class="strucfield">smoothedge</p>Set true for using a smooth edge in the calculations, or false (default) otherwise.</li>
  </ul>
</td>
</tr>
</table>

</br>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
Hscatter
</td>
<td class="parameter_values">
The matrix representation of the created Hamiltonian.
</td>
</tr>
<tr>
<td class="parameter_names">
wavefunction_indexes
</td>
<td class="parameter_values">
The list of the sites included in the antidot/dot system.
</td>
</tr>
</table>

  <br></br><br></br><br></br>




<?php $method_name = "CreateHandlesForMagneticField" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Creates function handles of the vector potentials and apply the magnetic filed in the ribbon Hamiltonians.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface.<?php echo $method_name; ?>() </p>


  <br></br><br></br><br></br>




<?php $method_name = "getHolePoints" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Determines the sites that should be cut off from the ribbon in order to create the hole. The hole_edge_points attribute <a href="index.php?location=ws_antidot#hole_edge_points&menu=structures">hole_edge_points</a> is also set to the calculated output.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">hole_edge_points = interface.<?php echo $method_name; ?>() </p>

<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
hole_edge_points
</td>
<td class="parameter_values">
A matrix with culomns $$z$$ (slab index), $$zpoints_min$$, $$zpoints_max$$ (lower and upper bounds of the sites in the slab $$z$$).
</td>
</tr>
</table>


  <br></br><br></br><br></br>




<?php $method_name = "create_scatter_GreensFunction" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Calculates the surface Greens function of the antidot. Sets the attributes <a href="#gfin">gfin</a> and <a href="#gfininv">gfininv</a> to the calculated results.</p>
  <p class="caption">Syntax</p>
  <p class="syntax">interface.<?php echo $method_name; ?>() </p>



  <br></br><br></br><br></br>





<?php $method_name = "GetWorkspace" ?>
<h2 class="methods"><?php echo $object_name; ?>::<?php echo $method_name; ?><a name="<?php echo $method_name; ?>"></a></h2>
  <p class="syntax">Return a class handle <a href="index.php?location=ws_antidot&menu=structures">ws_antidot</a> of the workspace of the interface <a href="index.php?location=antidot&menu=interfaces">antidot</a>. Changes in fields of the workspace are not always safe, since the reinitialization of the interface is not performed.</p>
  <p class="caption">Syntax</p>
  <p class="syntax"> interface.<?php echo $method_name; ?>(  ) </p>


<table>
<tr>
<th colspan="2"> Return </th>
</tr>
<tr>
<td class="parameter_names">
ret
</td>
<td class="parameter_values">
An instance of class handle <a href="index.php?location=ws_antidot&menu=structures">ws_antidot</a>.
</td>
</tr>
</table>
  <br></br><br></br><br></br>


