%    Minimal conductivity of graphene - based on Favago TPI v4.4
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
function MinimalConductivity( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

toCalculate = 1;
Continue    = 0;
outfilename = ['Ribbon_Conductivity',num2str( filenum ),'.mat'];
outputdir   = [];

height = 170;
Energy = 0.001; %In EV


ribbon_hamiltonian    = [];



Widthmin = 10;
Widthdelta = 20;
Widthnum = 30;%100;%200;
Widthvec = Widthmin:Widthdelta:Widthmin+Widthnum*Widthdelta;

% calculated conductance (and number of open channels) as a function of width/height
Conductivity = NaN(Widthnum+1,1); 
nyitott_csatornak = NaN(Widthnum+1,1);
DeltaC      = NaN(Widthnum+1,1);
aspectRatio = NaN(Widthnum+1,1); 

setOutputDir();
if Continue && toCalculate
  HandleForLoad = LoadFromFile( [outputdir,'/',outfilename] );
  Conductivity = HandleForLoad.LoadVariable('Conductivity', 'NoEmpty', 'On');
  aspectRatio = HandleForLoad.LoadVariable('aspectRatio', 'NoEmpty', 'On');
  DeltaC = HandleForLoad.LoadVariable('DeltaC', 'NoEmpty', 'Off');
  nyitott_csatornak = HandleForLoad.LoadVariable('nyitott_csatornak', 'NoEmpty', 'On');
  Widthvec = HandleForLoad.LoadVariable('Widthvec', 'NoEmpty', 'On');
  Energy = HandleForLoad.LoadVariable('Energy', 'NoEmpty', 'Off');
  HandleForLoad.ClearLoadedVariables(); 
end


CalculateTranspor();
Egeszabra();




%% Calculates transport
    function CalculateTranspor()
        
    
        if ~toCalculate 
             disp('Loading Calculated data')
            HandleForLoad = LoadFromFile( outfilename);
            Conductivity = HandleForLoad.LoadVariable('Conductivity', 'NoEmpty', 'On');
            aspectRatio = HandleForLoad.LoadVariable('aspectRatio', 'NoEmpty', 'On');
            DeltaC = HandleForLoad.LoadVariable('DeltaC', 'NoEmpty', 'On');
            Widthvec = HandleForLoad.LoadVariable('Widthvec', 'NoEmpty', 'On');
            Energy = HandleForLoad.LoadVariable('Energy', 'NoEmpty', 'Off');
            HandleForLoad.ClearLoadedVariable();
            
            setOutputDir();
            return
        end
        
        
        
        
         if Continue && toCalculate
           indexes2calc = find( isnan(Conductivity) );
        else
           indexes2calc = 1:length(Widthvec);
        end
        
        for idx = 1:length(indexes2calc)       
            width = Widthvec(indexes2calc(idx));
            ribbon_hamiltonian = Ribbon('width', width, 'height', height );     
        
            disp(' ----------------------------------')
            disp('Plotting conductivity:')
            Egeszabra();        
        
        
            disp(' ')
            disp(' ')
            disp(' ----------------------------------')
            disp('Calculating Conductance for the given aspect ratio')           
        

            
            tic
            [Conductivity_tmp,aspectRatio_tmp,~,nyitott_csatornak_num,DeltaC_tmp] = ribbon_hamiltonian.Transport( Energy, 'constant_channels', false );
            toc
            
            Conductivity(indexes2calc(idx)) = Conductivity_tmp;
            nyitott_csatornak(indexes2calc(idx)) = nyitott_csatornak_num(1);
            DeltaC(indexes2calc(idx)) = DeltaC_tmp;
            aspectRatio(indexes2calc(idx)) = aspectRatio_tmp;
            
            
            
            save( [outputdir,'/',outfilename], 'Conductivity', 'aspectRatio', 'nyitott_csatornak', 'Widthvec', 'Energy', 'DeltaC', 'height' );
            disp([ num2str(indexes2calc(idx)/length(Widthvec)*100),' % calculated of the conductance.'])
            disp( ' ' )
            
            
            
        end
                
    end



%% sets the output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end




    function Egeszabra( )
        

        
        figure1 = figure();
        
        fontsize = 9;
        
        %% Conductivity    
        
        %x_lim = [min(coordinates.x(:,1)) max(coordinates.x(:,1)) ];
        %x_lim = [0 rightside];
        
        theoretical_limit = 2/pi; %Theoretical minimal conductivity
        
        %y_lim = [0 2];
        
        indexek = logical( (abs(DeltaC) <= 0.5) & (aspectRatio~=0) & ~isnan(DeltaC) );
        x_lim = [0 max(aspectRatio(indexek))];
        
        if norm(x_lim) == 0
            x_lim = [0 1];
        end
        
        Position = [0.08 0.58 0.53 0.3];
        axes_cond = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,... 
                'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        
        numerics = plot(aspectRatio(indexek), Conductivity(indexek), 'Linewidth', 2, 'color', [0 0 0], 'Parent', axes_cond);       
        %plot(Widthvec(indexek), nyitott_csatornak(indexek), 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);
        try
            plot(aspectRatio(indexek), DeltaC(indexek), 'Linewidth', 1, 'color', [0 1 0], 'Parent', axes_cond);
        catch
            disp('No DeltaC data present')
        end
         plot( [0 max(aspectRatio(indexek))], ones(1,2)*theoretical_limit, 'r',  'Parent', axes_cond);
        
               
        
        
        %plot(Evec, nyitott_csatornak, 'Linewidth', 1, 'color', [1 0 0], 'Parent', axes_cond);    
        
         % Create xlabel
        %xlabel_position = [0 0 0];
        xlabel('W/L','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %xlabel_handle = get(axes_cond,'XLabel');  
        %set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

        % Create ylabel
        %ylabel_position = [0 0 0];
        ylabel('\sigma/\sigma_0','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes_cond);
        %ylabel_handle = get(axes_cond,'YLabel'); 
        %set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);
        
        
        %set(get(get(numerics,'Annotation'),'LegendInformation'), 'IconDisplayStyle','off'); % Exclude line from legend

        fig_legend = legend(axes_cond, {'conductivity', 'unitarity'});%, 'FontSize', fontsize, 'FontName','Times New Roman')
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')

        
        print('-depsc2', [outputdir,'/Minimal_Conductivity_', num2str(filenum),'.eps'])
        print('-dpdf', [outputdir,'/Minimal_Conductivity_', num2str(filenum),'.pdf'])
        if toCalculate
            close(figure1)
        end
        
        
    end




end
