function JosephsonCurrent( filenum )

if ~exist('filenum', 'var')
    filenum = 1;
end

    toCalculate = 1;
    Continue = 0;

    phi1 = 0;
    phi2vec = [];
    
    EF = 1e-6; % in eV
    Edb = 311;
    
    
    width = 1;
    height = 7; % the index of unit cells starts with 0
    B = 0; % in Tesla
    
    inputXML = 'Basic_Input_zigzag_leads_PRB_80_205408.xml';
    
    
    
    outfilename = ['Josephson_current_',num2str( filenum )];
    outputdir   = [];
    
    
    currentvec_continuum = [];
	currentvec_continuum_scattering = [];
	current_surf = [];
    currentvec_ABS = [];
    currentvec_NBS = [];
    DeltaPhi_vec = [];
    currentvec_1range = [];
    
    setOutputDir();    
    setvectors();
    
    if (Continue && toCalculate) || ~toCalculate
        HandleForLoad = LoadFromFile( [outputdir,'/',outfilename,'.mat'] );
        DeltaPhi_vec = HandleForLoad.LoadVariable('DeltaPhi_vec', 'NoEmpty', 'On');
        currentvec_continuum = HandleForLoad.LoadVariable('currentvec_continuum', 'NoEmpty', 'Off');
        currentvec_continuum_scattering = HandleForLoad.LoadVariable('currentvec_continuum_scattering', 'NoEmpty', 'Off');
        current_surf = HandleForLoad.LoadVariable('current_surf', 'NoEmpty', 'Off');
        currentvec_ABS = HandleForLoad.LoadVariable('currentvec_ABS', 'NoEmpty', 'Off');
        currentvec_NBS = HandleForLoad.LoadVariable('currentvec_NBS', 'NoEmpty', 'Off');
        currentvec_1range = HandleForLoad.LoadVariable('currentvec_1range', 'NoEmpty', 'Off');
        width = HandleForLoad.LoadVariable('width', 'NoEmpty', 'On');
        height = HandleForLoad.LoadVariable('height', 'NoEmpty', 'On');
        B = HandleForLoad.LoadVariable('B', 'NoEmpty', 'On');
        EF = HandleForLoad.LoadVariable('EF', 'NoEmpty', 'On');
        Edb = HandleForLoad.LoadVariable('Edb', 'NoEmpty', 'Off');
        HandleForLoad.ClearLoadedVariables(); 
    end
    
    currentVSDeltaPhi();
    EgeszAbra();
    
    
%% set vectors    
    function setvectors()
        phi_db = 40;
        deltaPhi = pi/(phi_db+1);     
        phi2vec = 0:deltaPhi:pi;      
        %phi2vec = pi/3;        
        
    end
    
%% currentVSDeltaPhi
    function currentVSDeltaPhi()
        
    if ~toCalculate
        return
    end
        
            
    DeltaPhi_vec = phi2vec - phi1;
    
    % continuum contribution
    SNSJosephson_handles = SNSJosephson( 'EF', EF, 'width', width, 'height', height, 'B', B, 'phi', [phi1, phi1+DeltaPhi_vec(1)], 'silent', true, 'filenameIn', inputXML);
    if isempty(currentvec_continuum_scattering)
        [currentvec_continuum_scattering, current_surf] = SNSJosephson_handles.CurrentCalc_continuum( DeltaPhi_vec, 'Edb', Edb );
        save( [outputdir,'/',outfilename,'.mat'], 'DeltaPhi_vec', 'currentvec_continuum', 'currentvec_continuum_scattering', 'current_surf', 'currentvec_ABS', 'currentvec_NBS', 'currentvec_1range', 'width', 'height', 'B', 'EF', 'Edb');
        EgeszAbra();
    end
    if isempty(currentvec_continuum)
        currentvec_continuum = SNSJosephson_handles.CurrentCalc_discrete( DeltaPhi_vec, 'range', 'CONT', 'Edb', Edb );
        save( [outputdir,'/',outfilename,'.mat'], 'DeltaPhi_vec', 'currentvec_continuum', 'currentvec_continuum_scattering', 'current_surf', 'currentvec_ABS', 'currentvec_NBS', 'currentvec_1range', 'width', 'height', 'B', 'EF', 'Edb');
        EgeszAbra();
    end
    
    % contribution from Andreev bound states
    if isempty(currentvec_ABS)
        currentvec_ABS  = SNSJosephson_handles.CurrentCalc_discrete( DeltaPhi_vec, 'range', 'ABS', 'Edb', Edb );
        save( [outputdir,'/',outfilename,'.mat'], 'DeltaPhi_vec', 'currentvec_continuum', 'currentvec_continuum_scattering', 'current_surf', 'currentvec_ABS', 'currentvec_NBS', 'currentvec_1range', 'width', 'height', 'B', 'EF', 'Edb');
        EgeszAbra();
    end
    
    % contribution from normal bound states
    if isempty(currentvec_NBS)
        currentvec_NBS  = SNSJosephson_handles.CurrentCalc_discrete( DeltaPhi_vec, 'range', 'NBS', 'Edb', Edb );
        save( [outputdir,'/',outfilename,'.mat'], 'DeltaPhi_vec', 'currentvec_continuum', 'currentvec_continuum_scattering', 'current_surf', 'currentvec_ABS', 'currentvec_NBS', 'currentvec_1range', 'width', 'height', 'B', 'EF', 'Edb');
        EgeszAbra();
    end
    
    if isempty(currentvec_1range)
        currentvec_1range = SNSJosephson_handles.CurrentCalc_discrete( DeltaPhi_vec, 'range', 'ALL', 'Edb', Edb );
        save( [outputdir,'/',outfilename,'.mat'], 'DeltaPhi_vec', 'currentvec_continuum', 'currentvec_continuum_scattering', 'current_surf', 'currentvec_ABS', 'currentvec_NBS', 'currentvec_1range', 'width', 'height', 'B', 'EF', 'Edb');
    end
    
    
 
    
    end


%% plotfunction
    function EgeszAbra()
        
        figure1 = figure();
        
        fontsize = 16;
          
        
                
        Position = [0.18 0.18 0.53 0.53];
        axes1 = axes('Parent',figure1, 'Position', Position,...
                'Visible', 'on',...
                'FontSize', fontsize,...                 'xlim', x_lim,...                    'ylim', y_lim,...                'XTick', XTick,...                'YTick', YTick,...
                'Box', 'on',...
                'FontName','Times New Roman');
        hold on; 
        
        % Create xlabel
        %xlabel_position = [0 0 0];
        xlabel('\Delta\phi/\pi','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes1);
        %xlabel_handle = get(axes_cond,'XLabel');  
        %set(xlabel_handle, 'Position', get(xlabel_handle, 'Position') + xlabel_position);

        % Create ylabel
        %ylabel_position = [0 0 0];
        ylabel('I','FontSize', fontsize,'FontName','Times New Roman', 'Parent', axes1);
        %ylabel_handle = get(axes_cond,'YLabel'); 
        %set(ylabel_handle, 'Position', get(ylabel_handle, 'Position') + ylabel_position);
        
        legend_labels = [];
        
        if ~isempty( currentvec_continuum )
            indexes = logical( currentvec_continuum );
            plot(DeltaPhi_vec(indexes)/pi, currentvec_continuum(indexes), 'b-', 'Parent', axes1)
			legend_labels{end+1} = 'CONT';
        end

        if ~isempty( currentvec_continuum_scattering )
            indexes = logical( currentvec_continuum_scattering );
            plot(DeltaPhi_vec(indexes)/pi, currentvec_continuum_scattering(indexes), 'b--', 'Parent', axes1)
			legend_labels{end+1} = 'CONT S-matrix';
        end
        
        if ~isempty( currentvec_ABS )
            indexes = ~isnan( currentvec_ABS );
            plot(DeltaPhi_vec(indexes)/pi, currentvec_ABS(indexes), 'Linestyle', '-', 'Color', [0 0.83 0], 'Parent', axes1) 
			legend_labels{end+1} = 'Andreev bound states';
        end
        
        if ~isempty( currentvec_NBS )
            indexes = ~isnan( currentvec_NBS );
            plot(DeltaPhi_vec(indexes)/pi, currentvec_NBS(indexes), 'r-', 'Parent', axes1)
			legend_labels{end+1} = 'Normal bound states';
        end
        
        if (~isempty( currentvec_NBS )) && (~isempty( currentvec_continuum )) && (~isempty( currentvec_NBS ))
            total = currentvec_continuum+currentvec_ABS+currentvec_NBS;
            indexes = ~isnan( total );
            plot(DeltaPhi_vec(indexes)/pi, total, 'k-', 'Parent', axes1)
			legend_labels{end+1} = 'Total current';
        end          
        
        
        if ~isempty( currentvec_1range )
            indexes = ~isnan( currentvec_1range );
            plot(DeltaPhi_vec(indexes)/pi, currentvec_1range(indexes), 'c-', 'Parent', axes1)
			legend_labels{end+1} = 'ALL';
        end
        
               

        fig_legend = legend(axes1, legend_labels);
        set(fig_legend, 'FontSize', fontsize, 'FontName','Times New Roman', 'Box', 'off', 'Location', 'NorthEast')
    
    
        xlabel('\Delta\phi/\pi')
        ylabel('I')        
               

        print('-depsc2', [outputdir,'/',outfilename,'.eps'])
        close(figure1)
        
        
        
        
    end



%% sets the output directory
    function setOutputDir()
        resultsdir = 'results';
        mkdir(resultsdir );
        outputdir = resultsdir;        
    end

        
        
    
end
