%%    Favago Transport Programming Interface - LoadFromFile
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%  Returns an object providing interface to load variables from a file.
%%
function handles = LoadFromFile(filenev)

    
if exist(filenev, 'file')
    LoadedVariable = load(filenev);
else
    LoadedVariable = [];
end

handles.LoadVariable = @LoadVariable;
handles.ClearLoadedVariables = @ClearLoadedVariables;



%% LoadVariable
% Loads variables from the opened file.
function ret = LoadVariable(varname, varargin)
    
    p = inputParser;
    p.addParamValue('NoEmpty', 'Off');
    p.addParamValue('StructName', []);
    p.parse(varargin{:});      
    
    NoEmpty    = p.Results.NoEmpty;
    StructName = p.Results.StructName;
    
    if ~isempty(StructName)
        dfieldnames = fieldnames(LoadedVariable.(StructName));
        if ismember(varname, dfieldnames)
            ret = LoadedVariable.(StructName).(varname);
        else
            ret = [];
        end
        
        if isempty(ret) && strcmp(NoEmpty, 'On')
           error(['"',filenev,'" has no element called "',StructName,'.',varname,'", or it is empty.']);
        end
        
        return;
    end
    
    
    
    if isfield(LoadedVariable, varname)
        ret  = LoadedVariable.(varname);
    else
        ret = [];
    end
    
    if isempty(ret) && strcmp(NoEmpty, 'On')
        error(['"',filenev,'" has no element called "',varname,'", or it is empty.']);
    end        
end


%% ClearLoadedVariables
% Disposes the loaded file.
function ClearLoadedVariables()
    LoadedVariable = [];
end
%--------------------------------------------------------------------------




end % Global end
