%%    Favago Transport Programming Interface - CreateHamiltonians
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    Returns an interface to evaluate the Dyson equation and to calculate the scattering matrix.
%%
function handles = Transport_Interface(E, Opt, param, varargin)


OverlapIntegralsHandles = [];



G = [];
Smatrix = [];
nyitott_csatornak  = [];
Conductance_Matrix = [];
Dysonfunc = [];

CreateH = [];
PeierlsTransform = [];
Surface = [];

InitializeParameters();


handles.SurfaceGreenFunctionCalculator = @SurfaceGreenFunctionCalculator;
handles.ScatterCalc = @ScatterCalc;
handles.LeadCalc    = @LeadCalc;
handles.LeadSave    = @LeadSave;
handles.SmatrixCalc = @SmatrixCalc;
handles.Conduktance = @Conduktance;
handles.DysonEq     = @DysonEq;
handles.Read        = @MemberRead;
%handles.Write       = @MemberWrite;
handles.setEnergy   = @setEnergy;
handles.replaceLead = @replaceLead;
handles.CreateClone = @CreateClone;


%% ScatterCalc
% Calculates the effective (decimated) Hamiltonian of the scattering region.
function ScatterCalc()
        
    display('Creating Hamiltonian for Scattering region')
    if isempty(CreateH)
        CreateH = CreateHamiltonians(Opt, param);
    end
    CreateH.CreateScatterH();

    if Opt.magnetic_field == 1
        display('Applying magnetic field in scattering region')
        if isempty( PeierlsTransform )
            PeierlsTransform = Peierls(Opt, 'param', param);
        end
        PeierlsTransform.PeierlsTransform(CreateH, 'Hscatter');
    else
        PeierlsTransform = [];
    end

    if ~isempty(param.scatter.Overlap_in_Scatter) && param.scatter.Overlap_in_Scatter && ~(isempty(Opt.type_of_scanning) ) && ~Opt.Just_Create_Hamiltonians
        display('Applying overlap integrals in scattering region')
        OverlapIntegralsHandles.ApplyOverlapInScatter( CreateH, E )
    end

    % Deciamtion
    if Opt.Decimation >= 1  && ~Opt.Decimate_only_Dyson && ~Opt.Just_Create_Hamiltonians
        ido = cputime();
        display('Applying decimation procedure in scattering region')
        Decimation_Procedure = Decimation(Opt); 
        Decimation_Procedure(E, CreateH, 'Hscatter', 'kulso_szabfokok');
        display(['A decimalasnal eltelt ido:', num2str(cputime()-ido)]);
    else
        display('No decimation calculation is performed in the scattering region')
    end

end



%% LeadCalc
% Invokes the function SurfaceGreenFunctionCalculator for each lead.
function surf_ret = LeadCalc( varargin )
        
    p = inputParser;
    p.addParamValue('shiftLeads', zeros(length(param.Leads),1));
    p.addParamValue('potential', []);
    p.addParamValue('coordinates_shift', zeros(length(param.Leads),1) );
    p.addParamValue('gauge_field', [] );
    p.addParamValue('createCore', 0);
    p.addParamValue('SelfEnergy',false); % set true to calculate the self energy of the semi-infinite lead
    p.addParamValue('SurfaceGreensFunction', true );% set true to calculate the surface Greens function of the semi-infinite lead
    p.parse(varargin{:});
    
    shiftLeads             = p.Results.shiftLeads;
    potential              = p.Results.potential;
    coordinates_shift      = p.Results.coordinates_shift; % shifts coordinates by "coordinates_shift" times of a lattice vector.
    gauge_field            = p.Results.gauge_field; % if a valid function handle is given, it is used to perform gauge transformation on Lead Hamiltonians
    createCore             = p.Results.createCore;
    SelfEnergy             = p.Results.SelfEnergy;
    SurfaceGreensFunction  = p.Results.SurfaceGreensFunction;
        
    Surface = cell(length(param.Leads),1);
    display(' ')
    display('Creating surface Greens functions:')
    display(' ')
    idx = 1;
    while idx <= length(param.Leads)
        display('------------------------------')
        display(['Lead: ',num2str(idx)]) 
        Surface{idx} = SurfaceGreenFunctionCalculator(idx, 'shiftLead', shiftLeads(idx), 'potential', potential, 'coordinates_shift', coordinates_shift(idx), 'gauge_field', gauge_field, 'createCore', createCore, ...
            'SelfEnergy', SelfEnergy, 'SurfaceGreensFunction', SurfaceGreensFunction);
        display(' ')
        idx = idx + 1;
    end

    surf_ret = Surface;

end

%% DysonEq
% Invokes the Dyson equation. 
function Gret = DysonEq( varargin )

    display('------------------------------')

    p = inputParser;
    p.addParamValue('CustomDyson', []);        
    p.parse(varargin{:});

    if ~isempty( p.Results.CustomDyson )
        Dysonfunc = p.Results.CustomDyson;
    end
    
    if ~isempty( Dysonfunc )
        G = Dysonfunc();
    else
        G = [];
    end
    Gret = G;

end

%% Conduktance
% Calculates the conductance matrix from the scattering matrix
function C = Conduktance()
    display('calculating conductance matrix')

    LeadsNumber        = length(nyitott_csatornak);
    C = zeros(LeadsNumber, LeadsNumber);
    
    soridx            = 1;
    while soridx <= LeadsNumber
        oszlopidx = 1;
        while oszlopidx <= LeadsNumber      
            s                   = Smatrix((sum(nyitott_csatornak(1:soridx-1))+1:sum(nyitott_csatornak(1:soridx))), (sum(nyitott_csatornak(1:oszlopidx-1))+1:sum(nyitott_csatornak(1:oszlopidx))));
            if soridx == oszlopidx
                C(soridx,oszlopidx) = nyitott_csatornak(soridx) - trace(s'*s);
            else
                C(soridx,oszlopidx) = -trace(s'*s);      
            end
            oszlopidx           = oszlopidx +1;
        end    
        soridx = soridx +1;
    end
    
    Conductance_Matrix = C;


end
%% SmatrixCal
% Calculates the scattering matrix
function [S, M] = SmatrixCalc( )
    display('calculating S-matrix')
    M = GetM();

    idx = 1;
    Normamtx = cell(length(M),1);
    modusmtx_m  = cell(length(M),1);
    modusmtx_p  = cell(length(M),1);
    d_modusmtx_m  = cell(length(M),1);
    gyokvcsop_p = cell(length(M),1);
    gyokvcsop_m = cell(length(M),1);
    k_p = cell(length(M),1);
    k_m = cell(length(M),1);
    sort_tolerance = cell(length(M),1);
    evanescent_indexes_p = cell(length(M),1);
    evanescent_indexes_m = cell(length(M),1);
    
    while idx <= length(M)
        Normamtx{idx} = Surface{idx}.Read('Normamtx');
        modusmtx_p{idx} = Surface{idx}.Read('modusmtx_p');
        modusmtx_m{idx} = Surface{idx}.Read('modusmtx_m');
        d_modusmtx_m{idx} = inv(modusmtx_m{idx});%Surface{idx}.Read('d_modusmtx_m');
        vcsop_p = Surface{idx}.Read('vcsop_p');
        k_p{idx}  = Surface{idx}.Read('k_p');
        gyokvcsop_p{idx} = sqrt(abs(real(vcsop_p)) + imag(vcsop_p)*1i);
        vcsop_m = Surface{idx}.Read('vcsop_m');
        k_m{idx} = Surface{idx}.Read('k_m');
        gyokvcsop_m{idx} = sqrt(abs(real(vcsop_m)) + imag(vcsop_m)*1i);
        sort_tolerance{idx} = Surface{idx}.Read('sort_tolerance');
%         Surface{idx}.Clear('Normamtx');
%         Surface{idx}.Clear('modusmtx_p');
%         Surface{idx}.Clear('modusmtx_m');
%         Surface{idx}.Clear('d_modusmtx_m');
%         Surface{idx}.Clear('vcsop_p');
%         Surface{idx}.Clear('vcsop_m');
        idx = idx + 1;
    end
    
    % indexing out evanescent modes
    for idx = 1:length(M)
        tolerance = sort_tolerance{idx};
        evanescent_indexes_p{idx} = logical( abs(imag(k_p{idx})) > tolerance );
        evanescent_indexes_m{idx} = logical( abs(imag(k_m{idx})) > tolerance );
    end   

    NormamtxAll    = zeros(sum(M), sum(M));
    Modusmtx_pAll  = zeros(sum(M), sum(M));
    Modusmtx_mAll  = zeros(sum(M), sum(M));
    d_Modusmtx_mAll = zeros(sum(M), sum(M));
    Gyokvcsop_pAll = zeros(sum(M), 1);
    Gyokvcsop_mAll = zeros(sum(M), 1);
    k_pAll = zeros(sum(M), 1);
    k_mAll = zeros(sum(M), 1);
    evanescent_indexes_pAll = false(sum(M), 1);
    evanescent_indexes_mAll = false(sum(M), 1);

    for idx = 1:length(M)
        NormamtxAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = Normamtx{idx};
        Modusmtx_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = modusmtx_p{idx};
        Modusmtx_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = modusmtx_m{idx};
        d_Modusmtx_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)), sum(M(1:idx-1))+1:sum(M(1:idx)) ) = d_modusmtx_m{idx};
        Gyokvcsop_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = gyokvcsop_p{idx};
        Gyokvcsop_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = gyokvcsop_m{idx};
        k_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = k_p{idx};
        k_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = k_m{idx};
        evanescent_indexes_pAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = evanescent_indexes_p{idx};
        evanescent_indexes_mAll( sum(M(1:idx-1))+1:sum(M(1:idx)) ) = evanescent_indexes_m{idx};
        
        Normamtx{idx}    = [];
        modusmtx_p{idx}  = [];
        modusmtx_m{idx}  = [];
        gyokvcsop_p{idx} = [];
        gyokvcsop_m{idx} = [];
    end
    
    %d_Modusmtx_mAll = inv(Modusmtx_mAll); % dualis bazis a kimenokre 
    
    GN0 = G(1:sum(M),1:sum(M))*NormamtxAll;
    NormamtxAll = [];    
    %S   = diag(Gyokvcsop_mAll) * (d_Modusmtx_mAll*(GN0-eye(sum(M))) )*Modusmtx_pAll * diag(Gyokvcsop_pAll.^(-1));
    S   = (d_Modusmtx_mAll*(GN0-eye(sum(M))) )*Modusmtx_pAll;
    
    %sorting out evanescent modes
    S(:,evanescent_indexes_pAll) = [];
    for idx=1:length(M)
        M(idx) = length(find(~evanescent_indexes_p{idx}));
    end    
    S(evanescent_indexes_mAll,:) = []; 
    
    S = diag( Gyokvcsop_mAll(~evanescent_indexes_mAll) )*S*diag(Gyokvcsop_pAll(~evanescent_indexes_pAll).^(-1));
    
    
        
    
 nyitott_csatornak = M;
 Smatrix = S;

end

%% SurfaceGreenFunctionCalculator
% Calculates the surface Greens function of a Lead
function Surface_tmp = SurfaceGreenFunctionCalculator(idx, varargin)
    
    p = inputParser;
    p.addParamValue('createCore', 0);
    p.addParamValue('Just_Create_Hamiltonians', 0);
    p.addParamValue('shiftLead', 0);
    p.addParamValue('coordinates_shift', 0);
    p.addParamValue('potential',[]);
    p.addParamValue('Surface_tmp',[]);
    p.addParamValue('gauge_field', [] );% gauge field for performing gauge transformation
    p.addParamValue('SelfEnergy',false); % set true to calculate the self energy of the semi-infinite lead
    p.addParamValue('SurfaceGreensFunction', true );% set true to calculate the surface Greens function of the semi-infinite lead
    p.parse(varargin{:});
    createCore            = p.Results.createCore;
    Just_Create_Hamiltonians = p.Results.Just_Create_Hamiltonians;
    shiftLead             = p.Results.shiftLead;
    coordinates_shift     = p.Results.coordinates_shift;
    potential             = p.Results.potential;
    Surface_tmp           = p.Results.Surface_tmp;
    gauge_field           = p.Results.gauge_field; % gauge field for performing gauge transformation
    SelfEnergy            = p.Results.SelfEnergy;
    SurfaceGreensFunction = p.Results.SurfaceGreensFunction;
    
    
    
    if isempty(idx)
        Surface_tmp = Surface_Green_function(Opt, param,...
                                            'Way2Hamiltonian', CreateH, ...
                                            'Hanyadik_Lead', idx);
                                        
    elseif ~isempty( Surface_tmp )
        % do nothing if Surface_tmp is given
        Surface_tmp.Reset();
                                        
    elseif ~isempty(param.Leads{idx}.Lead_Orientation)     
        Surface_tmp = Surface_Green_function(Opt, param,...
                                            'Way2Hamiltonian', CreateH, ...
                                            'Hanyadik_Lead', idx,....
                                            'Lead_Orientation', param.Leads{idx}.Lead_Orientation);
    else
        Surface_tmp = Surface_Green_function(Opt, param,...
                                            'Way2Hamiltonian', CreateH, ...
                                            'Hanyadik_Lead', idx);
    end
    
    
    if createCore
        return
    end
                                        
    if Opt.Simple_Green_Function
        Surface_tmp.SurfaceGreen_simple(E);        
    else
    
        % creating Hailtonians
        display(['Creating Hamiltonians for lead'])
        Surface_tmp.CreateHamiltonians();
        Surface_tmp.ShiftCoordinates( coordinates_shift );
        if Opt.magnetic_field == 1
            display(['Applying magnetic field in lead'])
            
            % in superconducting lead one must not include nonzero magnetic field
            if ~Surface_tmp.isSuperconducting()
                PeierlsTransform.PeierlsTransformLeads(Surface_tmp);
            elseif ~isempty( gauge_field )
            	PeierlsTransform.gaugeTransformationOnLead( Surface_tmp, gauge_field );
            end
            
            if Opt.saveWithMagneticFiled
                Surface_tmp.SaveLeads()
            end
        end
        
        if abs(shiftLead) > 1e-6
            Surface_tmp.ShiftLead( shiftLead );
        end

        if ~isempty(potential)
    	    Surface_tmp.AddPotential(potential);
        end        
        
        if Just_Create_Hamiltonians
            return;
        end
        
        if isempty(Opt.type_of_scanning) 
            return;
        end
        
        
        % Atfedesi integralok aplikalasa
        if ~isempty(param.Leads{idx}.Overlap_in_Leads) && param.Leads{idx}.Overlap_in_Leads && ~(isempty(Opt.type_of_scanning)) && ~Opt.Just_Create_Hamiltonians
            display(['Applying overlap integrals for lead'])
            OverlapIntegralsHandles.ApplyOverlapInLeads( Surface_tmp, E, idx)
        end
        
        %Decimalas hatszogracs eseten
        if strcmp(Opt.Lattice_Type, 'Graphene') || strcmp(Opt.Lattice_Type, 'Graphene_Bilayer') || strcmp(Opt.Lattice_Type, 'Silicene')
           display(['Decimating the lead Hamiltonians'])
           if Opt.Decimation == 0
               decimation = 3;
           else
               decimation = Opt.Decimation;
           end
           Decimation_Procedure_Lead = Decimation(Opt, 'ForcedDecimation', decimation);
           Decimation_Procedure_Lead(E, Surface_tmp, 'Hamiltonian2Dec', 'kulso_szabfokok');  
           clear Decimation_Procedure_Lead;
        end
        
        % Trukkos sajatertek
        display(['Eigenvalues for lead'])
        Surface_tmp.TrukkosSajatertekek(E);
        
        %Csportsebessegek
        display(['Group velocities for lead'])
        if ~isempty(param.Leads{idx}.Overlap_in_Leads) && param.Leads{idx}.Overlap_in_Leads && ~(isempty(Opt.type_of_scanning))
            Surface_tmp.Group_Velocity('WithOverlap', OverlapIntegralsHandles);
        else
            Surface_tmp.Group_Velocity();
        end
        
        
        
        %Feluleti Green fuggveny
        if SurfaceGreensFunction
            Surface_tmp.SurfaceGreenFunction();
        end
        
        %SelfEnergy
        if SelfEnergy
            Surface_tmp.SelfEnergy();
        end

    end
    
    
end
%------------------------------------------------------------------

%% setEnergy
% Set the energy to be used in the calculations
    function setEnergy( Energy )
       E = Energy; 
    end


%% MemberRead
% Reads the value of an element in the interface.
    function ret = MemberRead(MemberName)
        if strcmp(MemberName, 'param')
            ret = param;
        elseif strcmp(MemberName, 'Opt')
            ret = Opt;            
         elseif strcmp(MemberName, 'Surface')
            ret = Surface;  
         elseif strcmp(MemberName, 'E')
            ret = E;             
         elseif strcmp(MemberName, 'M')
            ret = GetM();    
         elseif strcmp(MemberName, 'G')
            ret = G;             
        else
            disp(['No member to read with name ',MemberName]);
        end
    end

%--------------------------------------------------------------------------
%     function MemberWrite(MemberName, value)
%         if strcmp(MemberName, 'param')
%             param = value;
%             InitializeParameters();
%         elseif strcmp(MemberName, 'Opt')
%             Opt = value;  
%             InitializeParameters();
%         else
%             disp(['No member to write with name ',MemberName]);
%         end
%     end


%% replaceLead
% Adds/replaces a lead to/in the system.
    function replaceLead( Surface_tmp, idx )
        Surface{ idx } = Surface_tmp;   
        Opt.NofLeads = length(Surface);
    end

%% LeadSave
% Saves the Hamiltonians of each lead
    function LeadSave()        
       for idx = 1:length(Surface)
          Surface{idx}.SaveLeads();
       end 
    end

%% CreateClone
%   Creates a clone of the current interface.
    function ret = CreateClone()
        
        if isempty( CreateH )
            CreateH_loc = [];
        else
            CreateH_loc = CreateH.CreateClone();
        end
        
        if isempty( PeierlsTransform )
            PeierlsTransform_loc = [];
        else
            PeierlsTransform_loc = PeierlsTransform.CreateClone();
        end
        
        ret = Transport_Interface(E, Opt, param, ...
            'CreateH', CreateH_loc, ...
            'PeierlsTransform', PeierlsTransform_loc );
        
        for idx = 1:length(Surface)
            ret.replaceLead( Surface{idx}.CreateClone(), idx );
        end      
        
    end   


%% GetM
% Get the width of the leads
    function M = GetM()
        M = zeros(length(Surface),1);
        for idx = 1:length(Surface)
           M(idx) = Surface{idx}.Read( 'M' );            
        end  
        
%         if ~isempty(M)
%             param.Leads.M = M;
%         end
    end

% Initializing parameters
    function InitializeParameters()
        OverlapIntegralsHandles = OverlapIntegrals(); 
        
        G = [];
        Smatrix = [];
        nyitott_csatornak  = [];
        Conductance_Matrix = [];

        CreateH = [];
        PeierlsTransform = [];
        Surface = [];
        
        InputParsing();

        
    end

% displays output messages on the screen
    function display( message, nosilent )
        if ~Opt.Silent || exist('nosilent', 'var')
            disp( message )
        end
       
        if Opt.debug
            fid = fopen('debug.txt', 'a');
            fprintf(fid, [message,'\n']);
            fclose(fid);
        end
        
    end

% Input parsing
    function InputParsing()
        
        p = inputParser;
        p.addParamValue('CreateH', []);
        p.addParamValue('PeierlsTransform', []);
        
        p.parse(varargin{:});

        PeierlsTransform = p.Results.PeierlsTransform;
        CreateH = p.Results.CreateH;

    end


end
