%%    Favago Transport Programming Interface - ValidationFunctions
%    Copyright (C) 2009-2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    Provides an interface for validating variables for usage in functions.
%%
function handles = ValidationFunctions

GeneralMessages          = Messages();

handles.Validating_kx_and_ky = @Validating_kx_and_ky;
handles.ValidatingFieldNames = @ValidatingFieldNames;


%% Validating_kx_and_ky
% Checking mesh grids kx and ky.
    function res = Validating_kx_and_ky(kx, ky, varargin)
    % use varargin to indicate that kx and ky can be matrices
    
    res = 0;
    if ~isnumeric(kx) || ~isreal(kx) || ~( isscalar(kx) || ~isempty(varargin))
        GeneralMessages.BadInputType('kx','real number');
        kx = 0;
        res = 1;
    end
        
    if ~isnumeric(ky) || ~isreal(ky) || ~( isscalar(kx) || ~isempty(varargin))
        GeneralMessages.BadInputType('ky','real number');
        ky = 0; %#ok<NASGU>
        res = 1;
    end
        
    end


%% ValidatingFieldNames
% Checks the field names of a structure
    function res = ValidatingFieldNames(expected_fieldnames, structure_name, structure, varargin)
        p = inputParser;
        p.addParamValue('Isstruct', 'On');
        p.addParamValue('FieldNumber', 'On');
        p.addParamValue('Ismember', 'On');
        p.parse(varargin{:});
        
        Isstruct = p.Results.Isstruct;
        FieldNumber = p.Results.FieldNumber;
        Ismember = p.Results.Ismember;
        
        on = {'On','on'};
        
        Isstruct    = ValidatingArguments(Isstruct);
        FieldNumber = ValidatingArguments(FieldNumber);
        Ismember    = ValidatingArguments(Ismember);
        
        
        res = 0;
        if ismember(Isstruct,on) && ~isstruct(structure)
            warning('MATLAB:NotStructure',['"',structure_name,'" is not a structure!!!']);
            res = 1;
        end
        
        structure_fields = fieldnames(structure);
        
        if ismember(FieldNumber,on) && length(expected_fieldnames) ~= length(structure_fields)
            warning('MATLAB:BadStructure',['Structure "',structure_name,'" have wrong number of fields']);
            res = 1;
        end
        
        validation_idx = 1;
        while validation_idx <= length(expected_fieldnames)
            if ismember(Ismember,on) && ~ismember(expected_fieldnames{validation_idx}, structure_fields)
                IncorrectField(expected_fieldnames{validation_idx}, structure_name);
                res = 1;
            end            
            validation_idx = validation_idx + 1;            
        end
            
        
        function IncorrectField(name, name_of_structure)
            warning('MATLAB:BadField',['"',name,'"','is not a member of structure "',name_of_structure,'"!!!']);
        end
        
        
        function var = ValidatingArguments(var)
            values = {'On','on','off','Off'};
            if ~ismember(var,values)
                var = 'On';
            end
        end
        
        
    end



end
