%     Favago Transport Programming Interface - ws_Ribbon
%    Copyright (C) 2015 Peter Rakyta, Ph.D.
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
% **************************************************************************
%% Handle class containing the workspace of the intreface Ribbon
%
% *Stored data:*
%
%   CreateH           (handle)         Handle responsible for creating Hamiltonians for the scattering region.
%   FL_handles        (handle)         Handle for the "four leads" object
%   Surface_tmp       (handle)         Instance of "Surface Greens Function" object for the scattering region
%   Surface_interface     (handle)      Instance of "Surface Greens Function" object for the interface region between the leads and scattering region (only if necessary)
%   PeierlsTransform_Scatter   (handle)  Handle responsible for Peierls transformation in the scatterung region.
%   PeierlsTransform_Scatter   (handle)  Handle responsible for Peierls transformation in the scatterung region.
%   gauge_field                (handle)  Function handle for the gauge tansformation
%
%   filenameIn       (string)            input xml filename
%   filenameOut      (string)            output xml filename
%    silent     (logical)                if true, no output messages are print
        
%   width            (scalar)            width of the scattering region
%   height           (scalar)            height of the scattering region
%   shift           (scalar)             the shift of the coordinates of the sites ("shift" times the lattice vector).
%   phi = [[],[]];  (vector)             pair potential phases in the left (1) and right (2) superconductor
%   E               (scalar)
%   EF              (scalar)              The Fermi energy in eV
%
%   rCC             (scalar)              site-site distance in Angstrom
%   h               (scalar)              Plancks constant               
%   qe              (scalar)             electron charge  
%   tree                                 The xml tree     

classdef ws_Ribbon < handle
	properties
        CreateH                  = [];
        FL_handles               = [];
        Surface_tmp              = [];
        Surface_interface        = [];
        PeierlsTransform_Scatter = [];
        PeierlsTransform_Leads   = [];
        
        gauge_field              = [];  % function handle for the scalar field to transform the vector potential from Landauy to Landaux
        potential                = []; % function handle for potential
        
        filenameIn = [];
        filenameOut = [];
        silent     = false; % if true, no output messages are print
        
        width = [];%100; % site numbers
        height = []; % site numbers
        shift = 0; % the shift of the coordinates of the sites
        
        phi = [[],[]];   % pair potential phases in the left (1) and right (2) superconductor
        
        E = 0;
        EF = []; %The Fermi energy in eV
        
        PNpotential = 0; %The height ofthe p-n potential across the ribbon.
        PNeta = 0.2; %The width of the pn-potential step in units of the total width of the ribbon. ($$0 less PNeta less 1$$)

        rCC = 1.42; %In Angstrom
        h = 6.626e-34;
        qe = 1.602e-19;
        
        % xml tree
        tree = [];
	end


end
